#include "AMDiS.h"

using namespace std;
using namespace AMDiS;

// ===========================================================================
// ===== function definitions ================================================
// ===========================================================================

/** \brief
 * Dirichlet boundary function
 */
class G : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(G);

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  double operator()(const WorldVector<double>& x) const {
    return exp(-10.0*(x*x));
  }
};

/** \brief
 * RHS function
 */
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(F);

  /** \brief
   * Constructor
   */
  F(int degree, double sigma_, double k_, int dim_)
    : AbstractFunction<double, WorldVector<double> >(degree), sigma(sigma_), k(k_), dim(dim_)
  {};

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  double operator()(const WorldVector<double>& x) const {
    double r2 = x*x;
    double ux = exp(-10.0*r2);
    double ux4 = ux*ux*ux*ux;
    return sigma*ux4 - k*(400.0*r2 - 20.0*dim)*ux;
  }

private:
  double sigma;
  double k;
  int dim;
};

/** \brief
 * Needed for zero order term.
 */
class CQPFct : public AbstractFunction<double, double>
{
public:
  MEMORY_MANAGED(CQPFct);

  CQPFct() : AbstractFunction<double, double>(3) {};

  /** \brief
   * Constructor.
   */
  CQPFct(double sigma_) : sigma(sigma_) {};

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  double operator()(const double& x) const {
    return sigma * x * x * x;
  }

private:
  double sigma;
};


// ===========================================================================
// ===== class NonLin ========================================================
// ===========================================================================

/** \brief
 * Non linear problem.
 */
class NonLin : public ProblemNonLinScal
{
public:
  /** \brief
   * Constructor.
   */
  NonLin()
    : ProblemNonLinScal("nonlin"),
      sigma(1.0)
  {};

  /** \brief
   * Returns \ref sigma.
   */
  double getSigma() { return sigma; };

  /** \brief
   * Sets \ref sigma.
   */
  void setSigma(double sig) { sigma = sig; };

private:
  /** \brief
   * Stefan-Bolzmann constant.
   */
  double sigma;
};

// ===========================================================================
// ===== main program ========================================================
// ===========================================================================

int main(int argc, char* argv[])
{
  FUNCNAME("main");

  // ===== check for init file =====
  TEST_EXIT(argc == 2)("usage: nonlin initfile\n");

  // ===== init parameters =====
  Parameters::init(false, argv[1]);
  
  // ===== needed constants =====
  double k = 1.0;
  double factor1 = 4.0;
  double factor2 = -1.0;

  // ===== create and init non linear problem =====
  NonLin nonlin;
  nonlin.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo *adaptInfo = NEW AdaptInfo("nonlin->adapt", 1);

  // === create adapt ===
  AdaptStationary *adapt = NEW AdaptStationary("nonlin->adapt",
					       &nonlin,
					       adaptInfo);


  nonlin.setSigma(1.0);

  // ===== create operators =====
  Operator *nonLinOperator0 = NEW Operator(Operator::MATRIX_OPERATOR | 
					   Operator::VECTOR_OPERATOR, 
					   nonlin.getFESpace());

  nonLinOperator0->setUhOld(nonlin.getSolution());

  nonLinOperator0->addZeroOrderTerm
    (NEW VecAtQP_ZOT(NULL, NEW CQPFct(nonlin.getSigma())));

  nonlin.addMatrixOperator(nonLinOperator0, &factor1, &factor1);
  nonlin.addVectorOperator(nonLinOperator0);

  Operator *nonLinOperator2 = NEW Operator(Operator::MATRIX_OPERATOR | 
					   Operator::VECTOR_OPERATOR,
					   nonlin.getFESpace());

  nonLinOperator2->setUhOld(nonlin.getSolution());

  nonLinOperator2->addSecondOrderTerm(NEW FactorLaplace_SOT(&k));

  nonlin.addMatrixOperator(nonLinOperator2);
  nonlin.addVectorOperator(nonLinOperator2);


  int degree = nonlin.getFESpace()->getBasisFcts()->getDegree();

  Operator* rhsFunctionOperator = 
  NEW Operator(Operator::VECTOR_OPERATOR, nonlin.getFESpace());
  rhsFunctionOperator->addZeroOrderTerm
    (NEW CoordsAtQP_ZOT(NEW F(degree, 
			      nonlin.getSigma(), 
			      k, 
			      nonlin.getMesh()->getDim())
			)
     );
  
  nonlin.addVectorOperator(rhsFunctionOperator, &factor2, &factor2);

  // ===== add boundary conditions =====
  nonlin.addDirichletBC(DIRICHLET, NEW G);

  // ===== start adaption loop =====
  adapt->adapt();

  // ===== print solution =====
  nonlin.writeFiles(adaptInfo, true);

  return 0;
}





